/* Row.cc contains the definitions of non-trival Row operations.

	Written by: Joel C. Adams, Spring 1994 at Calvin College.
	Written for: Hands On C++, Lab 14.
---------------------------------------------------------------------*/

#include "Row.h"

#include <stdlib.h> // provides exit()

/* --- operator<< displays a Row.

	Receive: Out, an ostream object;
		R, a Row object.
	Output: The values in R, onto Out.
	Return(parameters): Out, containing the values of R;
	Return(function): Out, for chaining.
-------------------------------------------------------------------*/

ostream& operator<<(ostream& Out, const Row& R)
{
  for (int i = 0; i < R.NumValues_; i++)
    Out << R.Alpha[i] << ' ';

  return Out;
}


/* --- operator>> fills a Row with input values.

	Receive: In, an istream object;
		R, a Row object.
	Input: A sequence of values from In, storing them in R.
	Return(parameters): In, the input values extracted;
			R, containing the input values.
	Return(function): In, for chaining.
-------------------------------------------------------------------*/

istream& operator>>(istream& In, Row& R)
{
  char Separator = ' ';
  R.NumValues_ = 0;

  for (int i = 0; (i < R.Capacity()) && (Separator != '\n'); i++)
    {
      R.Alpha[i] = In.get();
      Separator = In.get();
      R.NumValues_++;
    }

  return In;
}


/* --- Operator [] provides the Row subscript operation.

	Receive: A Row object;
			Index, an unsigned value.
	Return: (A reference to) the value whose index is Index within
		the Row object.

  Note: Two versions are provided, one for const Rows and one for
	non-const Rows.
----------------------------------------------------------------------*/

const char Row::operator[] (unsigned Index) const
{
  if (Index >= NumValues_)
  {
	cerr << "\n*** Row subscript is out of range!\n";
	exit(-1);
  }

  return Alpha[Index];
}

char Row::operator[] (unsigned Index)
{
  if (Index < NumValues_)
  {
	cerr << "\n*** Row subscript is out of range!\n";
	exit(-1);
  }

  return Alpha[Index];
}

