/* Row.h contains the declarations of class Row.

	Written by: Joel C. Adams, Spring 1994 at Calvin College.
	Written for: Hands On C++, Lab 14.
---------------------------------------------------------------*/

#ifndef ROW
#define ROW

#include <iostream.h>

class Row
{
  enum {Capacity_ = 6};	// the # elements in a Row
  char Alpha [ Capacity_ ];	// the array data member
  unsigned NumValues_;	// the number of values in the Row

 public:

  /* --- The Row constructor initializes a Row object.

	Precondition: A Row object has been declared.
	Postcondition: The Row object has been initialized as empty.
  ----------------------------------------------------------------*/
  Row() { NumValues_ = 0; }


  /* --- Capacity() returns the total capacity of a Row object.

	Receive: A Row object.
	Return: The maximum number of values that Row can hold.
  -----------------------------------------------------------------*/
  unsigned Capacity() const { return Capacity_;}


  /* --- NumValues() indicates the number of values in a Row.

	Receive: A Row object.
	Return: The number of values in that Row.
  -------------------------------------------------------------------*/
  unsigned NumValues() const { return NumValues_;}


  /* --- Operator [] provides the Row subscript operation.

	Receive: A Row object;
			Index, an unsigned value.
	Return: (A reference to) the value whose index is Index within
		the Row object.

  Note: Two versions are provided, one for const Rows and one for
	non-const Rows.
  --------------------------------------------------------------------*/
  const char operator[] (unsigned Index) const;
  char operator[] (unsigned Index);

  /* --- operator<< displays a Row.

	Receive: Out, an ostream object;
		R, a Row object.
	Output: The values in R, onto Out.
	Return(parameters): Out, containing the values of R;
	Return(function): Out, for chaining.
  -----------------------------------------------------------------*/
  friend ostream& operator<<(ostream& Out, const Row& R);


  /* --- operator>> fills a Row with input values.

	Receive: In, an istream object;
		R, a Row object.
	Input: A sequence of values from In, storing them in R.
	Return(parameters): In, the input values extracted;
			R, containing the input values.
	Return(function): In, for chaining.
  -----------------------------------------------------------------*/
  friend istream& operator>>(istream& In, Row& R);


};

#endif

