/* RealArray.cc contains the definitions for class RealArray,
    an array-based class whose array is allocated at run-time.

   Written by:
   Written for: Hands On C++, Lab 15 (Part II).
----------------------------------------------------------------*/

#include "RealArray.h"

#include <stdlib.h>  // provides exit()
#include <assert.h>  // provides assert()


/* --- The class constructor builds a RealArray of a given Size.

   Precondition: A RealArray has been declared.
   Receive: Size, an (unsigned) integer.
   Postcondition: A RealArray of Size elements has been constructed.
-------------------------------------------------------------------*/

RealArray::RealArray(unsigned Size)
{
	// insert constructor statements here
}


/* --- Operator = provides RealArray assignment.

   Receive: A RealArray object;
			Original, the RealArray being assigned.
   Pass back: The RealArray object, as a copy of Original.
   Return(function): A reference to the RealArray object (for chaining).
-------------------------------------------------------------------------*/

RealArray& RealArray::operator=(const RealArray& Original)
{
	// insert assignment statements here
}


/* --- The copy constructor builds a RealArray copy, when needed.

   Precondition: A copy of a RealArray is needed.
   Receive: Original, (a reference to) the RealArray to be copied.
   Postcondition: A copy of Original has been constructed.
-----------------------------------------------------------------*/

// Copy constructor definition goes here



/* --- Operator [] provides the subscript operation for a RealArray.

   Receive: A RealArray object;
            Index, an unsigned integer.
   Return: (A reference to) the array element whose index is Index.

   Note: Two distinct versions of operator[] are provided, one for
          const RealArrays and one for non-const RealArrays.
--------------------------------------------------------------------*/

double& RealArray::operator[] (unsigned Index)
{
  if (Index >= Length_)
    {
      cerr << "\n*** RealArray[] received invalid index: "
           << Index << "\n\n";
      exit (-1);
    }

  return ArrayPtr[Index];
}

const double& RealArray::operator[] (unsigned Index) const
{
  if (Index >= Length_)
    {
      cerr << "\n*** RealArray[] received invalid index: "
           << Index << "\n\n";
      exit (-1);
    }

  return ArrayPtr[Index];
}

/* --- Operator << provides iostream output for RealArrays.

   Receive: Out, an ostream;
            RArray, a RealArray object.
   Output: The values of RArray, onto Out.
   Pass back: Out, containing the output values.
   Return: Out, for chaining.
-------------------------------------------------------------*/

ostream& operator<<(ostream& Out, const RealArray& RArray)
{
  for (int i = 0; i < RArray.Length_; i++)
    Out << RArray.ArrayPtr[i] << ' ';

  return Out;
}


/* --- Operator >> provides iostream input for RealArrays.

   Receive: In, an istream;
            RArray, a RealArray object.
   Input: A sequence of values, extracted from In.
   Pass back: In, without the input values;
              RArray, containing the input values.
   Return: In, for chaining.
-------------------------------------------------------------*/

istream& operator>>(istream& In, RealArray& RArray)
{
  for (int i = 0; i < RArray.Length_; i++)
    In >> RArray.ArrayPtr[i];

  return In;
}

