/* RealArray.h contains the declarations for class RealArray,
    an array-based class whose array is allocated at run-time.

   Written by:
   Written for: Hands On C++, Lab 15 (Part II).
----------------------------------------------------------------*/

#ifndef REALARRAY
#define REALARRAY

#include <iostream.h>

class RealArray
{


  unsigned
    Length_;

 public:


  /* --- The class constructor builds a RealArray of a given Size.

   Precondition: A RealArray has been declared.
   Receive: Size, an (unsigned) integer.
   Postcondition: A RealArray of Size elements has been constructed.
   -------------------------------------------------------------------*/
  RealArray(unsigned Size);


  /* --- The copy constructor builds a RealArray copy, when needed.

   Precondition: A copy of a RealArray is needed.
   Receive: Original, (a reference to) the RealArray to be copied.
   Postcondition: A copy of Original has been constructed.
   -----------------------------------------------------------------*/
   // Copy constructor declaration goes here


  /* --- The class destructor reclaims the run-time memory of a RealArray
   at the end of its lifetime.

   Precondition: The lifetime of a RealArray is over.
   Postcondition: Its run-time allocated memory has been reclaimed.
   -----------------------------------------------------------------------*/
   // destructor declaration goes here


  /* Length() returns the length of a RealArray.

     Receive: A RealArray object.
     Return: The length of its run-time allocated array.
  -------------------------------------------------------*/
  unsigned Length() const { return Length_; }


  /* --- Operator = provides RealArray assignment.

   Receive: A RealArray object;
            Original, the RealArray being assigned.
   Pass back: The RealArray object, as a copy of Original.
   Return(function): A reference to the RealArray object (for chaining).
   -------------------------------------------------------------------------*/
  RealArray& operator=(const RealArray& RArray);


  /* --- Operator [] provides the subscript operation for a RealArray.

   Receive: A RealArray object;
            Index, an unsigned integer.
   Return: (A reference to) the array element whose index is Index.

   Note: Two distinct versions of operator[] are provided, one for
          const RealArrays and one for non-const RealArrays.
   --------------------------------------------------------------------*/
  double& operator[] (unsigned Index);
  const double& operator[] (unsigned Index) const;


  /* --- Operator << provides iostream output for RealArrays.

   Receive: Out, an ostream;
            RArray, a RealArray object.
   Output: The values of RArray, onto Out.
   Pass back: Out, containing the output values.
   Return: Out, for chaining.
   -------------------------------------------------------------*/
  friend ostream& operator<<(ostream& Out, const RealArray& RArray);


  /* --- Operator >> provides iostream input for RealArrays.

   Receive: In, an istream;
            RArray, a RealArray object.
   Input: A sequence of values, extracted from In.
   Pass back: In, without the input values;
              RArray, containing the input values.
   Return: In, for chaining.
   -------------------------------------------------------------*/
  friend istream& operator>>(istream& In, RealArray& RArray);

};

#endif

